#ifndef __HIERACHY_H__
#define __HIERACHY_H__

#include "config.h"
#include "rootnode.h"

IceWM_CM_BEGIN_NAMESPACE

#define TREE_NODE(ctree,node) \
    ((TreeNode*) gtk_ctree_node_get_row_data (ctree, node))
    
class Hierachy {
public:
    Hierachy				(void) { }
    
private:
    GtkCTree*				widget;
    GtkCTreeNode*			current_node;
    
    class RootNode*			menu;
    class RootNode*			toolbar;
    class RootNode*			keys;
    class RootNode*			prefs;
    class RootNode*			winopts;
    class RootNode*			themes;
    
    static void delete_tree_node	(TreeNode*		node);
    
public:
    GtkWidget* init			(void);
    GtkCTreeNode* current		(void) { return current_node; }
    TreeNode* tree_node			(GtkCTreeNode*		node);
    gint row_height			(void);
    
    GtkCTreeNode* insert_tree_node	(TreeNode*		node,
    					 GtkCTreeNode*		parent = 0,
    					 GtkCTreeNode*		silbing = 0);
    void insert_interactively		(TreeNode*		node);
    void remove_tree_node		(TreeNode*		node = 0);
    void remove_tree_node_children	(TreeNode*		node = 0);
    void update_tree_node		(GtkCTreeNode*		row = 0);
    void update_tree_node		(TreeNode*		node,
    					 GtkCTreeNode*		row = 0);
    bool prev_node_is_moveable		(GtkCTreeNode*		ctree_node = 0);
    bool next_node_is_moveable		(GtkCTreeNode*		ctree_node = 0);
    void move_upwards			(GtkCTreeNode*		ctree_node = 0);
    void move_downwards			(GtkCTreeNode*		ctree_node = 0);

    void load				(RootNode::Location	location,
					 const char*		custom_file = 0);
    void store				(RootNode::Location	location,
					 const char*		custom_file = 0);
    
private:
    static void tree_select_row		(GtkCTree*		ctree,
					 GtkCTreeNode*		row,
					 gint			column,
					 Hierachy*		hierachy);
    static void tree_unselect_row	(GtkCTree*		ctree,
					 GtkCTreeNode*		row,
					 gint			column,
					 Hierachy*		hierachy);
    static void tree_move		(GtkCTree*		ctree,
					 GtkCTreeNode*		node,
					 GtkCTreeNode*		new_parent,
					 GtkCTreeNode*		new_silbing,
					 Hierachy*		hierachy);
};

inline TreeNode* Hierachy::tree_node	(GtkCTreeNode*		node) {
    return (TreeNode*) gtk_ctree_node_get_row_data (widget, node);
}

inline gint Hierachy::row_height	(void) {
    return GTK_CLIST (widget)->row_height;
}

IceWM_CM_END_NAMESPACE

#endif
