#include "lexer.h"

IceWM_CM_NAMESPACE

Lexer::Lexer	(istream&	input,
		 const char*	filename)
  : input (input) {
    line = 0;
    column = 0;
    top_char = '\n';
    keep_char = false;
    state = 0;
    this->filename = filename ? g_strdup (filename) : NULL;
}

Lexer::~Lexer (void) {
    g_free (filename);
}

char Lexer::get_next_char (void) {
    if (keep_char)
        keep_char = false;
    else {
        if ('\n' == top_char) {
            column = 1; ++line;
        } else
            ++column;

        top_char = input.get ();
    }	
    
    return top_char;
}

Lexer::Token Lexer::get_next_token (void) {
    if (eof ())
        token = END_OF_FILE;
    else {
/* consume whitespaces if requested */	
	const char* ws = whitespaces ();
	const char* lc = line_comments ();
	bool in_comment = false;
	
        if (NULL != ws || NULL != lc)
	    while (strchr (ws, get_next_char ()) ||
		   (in_comment|= (NULL != strchr (lc, curr_char ()))))
                in_comment&= (curr_char () != '\n');
        else
	    get_next_char ();
	    
/* parse quoted string if requested */	
	const char* qms;
        if (NULL != (qms = quotation_marks ()) &&
	    strchr (qms, curr_char ())) {
	    char qm;
		
            strbuf = "";
            qm = curr_char ();
            while (qm != get_next_char () && !eof ())
                strbuf+= curr_char ();

            if (eof ())
	        token = END_OF_FILE;
	    else {
	        value.string = strbuf.c_str ();
	        token = STRING;
            }
        } else {
/* token has to be something like an identifier */
            for (strbuf = "";
	         !(strchr (delimiters (), curr_char ()) || eof ()); 
		 get_next_char ())
		 strbuf+= curr_char ();

	    if (!(strbuf.length () || strchr (ws, curr_char ())))
		strbuf = curr_char ();
		
            if (strbuf.length ()) {
	        if (single_char_tokens () && (strbuf.length () == 1))
		    token = (Token) strbuf [0];
                else {
		    keep_curr_char (); /* it is a delimiter token... */
		    if (lookup_symbols () && 
		        lookup_symbol (strbuf.c_str ())) {
		        value.symbol = symbol->second;
		        token =  symbol_value_as_token ()
			      ? (Token) value.token : SYMBOL;
                    } else {
		        value.string = strbuf.c_str ();
		        token = identifiers_as_string () ? STRING : IDENTIFIER;
                    }
                }
	    } else
		token = END_OF_FILE;
	}
    }
    return token;
}

const char* Lexer::get_token_name (Token	token,
				   const int	state) {
    switch (token) {
    case END_OF_FILE:
        return _("<End Of File>");
    case STRING:
        return _("string");
    case IDENTIFIER:
        return _("identifier");
    case SYMBOL:
        return _("symbol");
    default:
        if (single_char_tokens () && token < LAST) {
	    static char single_char_token [] = "' '";
	    single_char_token [1] = (char) token;
	    return single_char_token;
	} else if (lookup_symbols ())
	    for (SymbolTable::iterator i = symtab [state].begin ();
	         i != symtab [state].end (); i++)
		 if (i->second == (const void*) token)
		     return i->first;
    }

    return _("<unknown token>");
}
