#ifndef __MAIN_H__
#define __MAIN_H__

#include "config.h"
#include "equal.h"
#include "hierachy.h"
#include "toolbar.h"

#include <hash_map>

IceWM_CM_BEGIN_NAMESPACE

class Main {
public:
    enum StatusContext {
        CTXT_NODE_DESCRIPTION = 1
    };
    
    Toolbar				toolbar;
    Hierachy				hierachy;
    
    static Main				app;
    char*				cfgdir;
    char*				libdir;

private:
    typedef hash_map<const char*, TreeNode::Icon, hash<const char*>, eqstr>
    					IconCache;

    GtkWidget*				frame;
    GtkWidget*				editor_tank;
    GtkWidget*				titlebar;
    GtkStatusbar*			status;
    GtkWidget*				about_page;
    
    IconCache				icon_cache;
    
public:
    int run				(int&		argc,
					 char**&	argv);
    static void quit			(void);
    const TreeNode::Icon* load_icon	(const char*	name);
    void push_message			(StatusContext	context,
    					 const char*	message);
    void pop_message			(StatusContext	context);
    void set_editor			(const char*	title,
    					 const char*	description,
    					 GtkWidget*	new_editor = NULL);
					 
    char* find_file			(const char*	filename,
    					 bool		lookup_private = true,
    					 bool		lookup_config = true,
    					 bool		lookup_library = true,
    					 const char*	paths [] = NULL,
    					 const char*	prefixes [] = NULL,
    					 const char*	suffixes [] = NULL);
    static ostream& write_header	(ostream&	os,
    					 const char*	filename);

private:
    bool parse_args			(int&		argc,
					 char**&	argv);
    void build_gui			(void);
    GtkWidget* create_about_page	(void);
    
};

IceWM_CM_END_NAMESPACE

#endif
