#ifndef __MENU_NODE_H__
#define __MENU_NODE_H__

#include "propertynode.h"
#include <iostream>

IceWM_CM_BEGIN_NAMESPACE

class MenuNode : public PropertyNode {
private:
    static GtkWidget*			editor;
    
public:
    enum PropId {
        LABEL, ICON, COMMAND, RESTART,
	PROPERTY_COUNT
    };
    
    static PropDef			prop_df [PROPERTY_COUNT];

protected:
    static GtkWidget*			prop_ed [PROPERTY_COUNT + 2];
    static GtkWidget*			bt_icon;

    virtual PropDef& prop_def		(const unsigned	index);
    virtual GtkWidget*& prop_editor	(const unsigned	index);
    
    GtkWidget* create_editor		(void);
    GtkWidget* create_top_panel		(void);
    virtual GtkWidget* get_top_panel	(void) = 0;
    const char* icon_name		(void);
					 
    static void label_changed		(GtkEditable	*editable,
    					 const unsigned index);
    static void icon_changed		(GtkEditable	*editable,
    					 const unsigned index);
					 
public:
    virtual const char* get_label	(void) { return get_string (LABEL); }
    virtual const Icon* get_icon	(void);
    virtual GtkWidget* get_editor	(void);
    
    virtual bool is_moveable		(void) { return true; }
    virtual bool is_removeable		(void) { return true; }
    virtual InsertionInfo* insertion_list
    					(void);
					
    virtual ostream& store		(ostream&	os) = 0;
    virtual void synchronize_content	(void) { ungrab_current (this); }
};

inline PropertyNode::PropDef& MenuNode::prop_def
					(const unsigned	index) {
    g_assert (index < prop_count ());
    return prop_df [index];
}

inline GtkWidget*& MenuNode::prop_editor
					(const unsigned	index) {
    g_assert (index < prop_count ());
    return prop_ed [index];
}

inline const char* MenuNode::icon_name	(void) {
    const char* icon = get_string (ICON);
    return (icon && *icon ? icon : "-");
}
    
inline const TreeNode::Icon* MenuNode::get_icon
					(void) {
    return (prop_count () > ICON ? PropertyNode::get_icon_from_prop (ICON)
    				 : NULL);
}
    
IceWM_CM_END_NAMESPACE

#endif
