#ifndef __MENU_ROOT_H__
#define __MENU_ROOT_H__

#include "config.h"
#include "rootnode.h"
#include "lexer.h"

IceWM_CM_BEGIN_NAMESPACE

class MenuRoot : public RootNode {
protected:
    enum Token {
        MENU =				Lexer::LAST,
        PROG,
        RESTART,
        SEPARATOR
    };
    
public:
    static InsertionInfo		insertion_info [];

    virtual const char* get_label	(void) { return _("Menu"); }
    virtual bool is_leaf		(void) { return false; }
    virtual void load			(Location	location,
			 		 const char*	custom_file = 0);
    virtual void store			(Location	location,
			 		 const char*	custom_file = 0);
    virtual InsertionInfo*
			insertion_list	(void) { return insertion_info; }

protected:
    void parse_menu_content		(Lexer&		lexer,
    					 GtkCTreeNode*	root_node,
    					 bool		submenu = false);
    void parse_menu			(Lexer&		lexer,
    					 GtkCTreeNode*	root_node);
    void parse_program			(Lexer&		lexer,
    					 GtkCTreeNode*	root_node,
    					 bool		restart = false);
    virtual const char* basename	(void) { return "menu"; }
    
    static void insert_new_application	(GtkWidget*	widget,
    					 gpointer	user_data);
    static void insert_new_submenu	(GtkWidget*	widget,
    					 gpointer	user_data);
    static void insert_new_separator	(GtkWidget*	widget,
    					 gpointer	user_data);
    static ostream& write_ident		(ostream&	os,
    					 int		ident);
};

inline ostream& MenuRoot::write_ident	(ostream&	os,
					 int		ident) {
    for (int n = 0; n < ident; ++n) os << ' ' << ' ';
    return os;
}

IceWM_CM_END_NAMESPACE

#endif
