#include "menuentry.h"
#include "../main.h"

#include "menuroot.h"
#include "namedmenuentry.h"
#include "submenu.h"
#include "program.h"

#include <gtk90degree.h>

IceWM_CM_NAMESPACE

GtkWidget* MenuEntry::editor = 0;
GtkWidget* Submenu::top_panel = 0;
GtkWidget* Program::top_panel = 0;
GtkWidget* Program::en_command = 0;
GtkWidget* Program::cb_restart = 0;

GtkWidget* NamedMenuEntry::create_top_panel (void) {
    GtkTable  *top_panel;
    GtkWidget **children;
    
    top_panel = GTK_TABLE (gtk_table_new (2, 3, FALSE));
    gtk_table_set_col_spacing  (top_panel, 1, 4);
    children = g_new0 (GtkWidget*, 3);
    
    gtk_table_attach (top_panel, new_label (_("Label:")),
                      1, 2, 1, 2, GTK_FILL, GTK_FILL, 0, 1);
    gtk_table_attach (top_panel, children [EN_LABEL] = gtk_entry_new (),
                      2, 3, 1, 2, GTK_FILL_AND_EXPAND, GTK_FILL, 0, 1);
		      
    gtk_table_attach (top_panel, new_label (_("Icon:")),
                      1, 2, 2, 3, GTK_FILL, GTK_FILL, 0, 1);
    gtk_table_attach (top_panel, children [EN_ICON] = gtk_entry_new (),
                      2, 3, 2, 3, GTK_FILL_AND_EXPAND, GTK_FILL, 0, 1);
    gtk_table_attach (top_panel, children [BT_ICON] = gtk_button_new (), 
                      3, 4, 1, 4, GTK_FILL, GTK_SHRINK, 4, 0);
    gtk_widget_set_usize (children [BT_ICON], 56, 56);
		      
    gtk_signal_connect (GTK_OBJECT (children [EN_LABEL]), "changed", 
                        GTK_SIGNAL_FUNC (name_changed), (gpointer) EN_LABEL);
    gtk_signal_connect (GTK_OBJECT (children [EN_ICON]), "changed", 
                        GTK_SIGNAL_FUNC (name_changed), (gpointer) EN_ICON);
    gtk_object_set_data_full (GTK_OBJECT (top_panel), "children",
    			      children, &g_free);

    return GTK_WIDGET (top_panel);
}

void NamedMenuEntry::fill_top_panel (GtkWidget* top_panel) {
    GtkWidget** children;
    
    gtk_object_set_data (GTK_OBJECT (top_panel), "NamedMenuEntry*", this);
    children = get_children (top_panel);

    update_icon_button (children [BT_ICON]);
    gtk_entry_set_text (GTK_ENTRY (children [EN_LABEL]), label);
    gtk_entry_set_text (GTK_ENTRY (children [EN_ICON]), icon);
}

void NamedMenuEntry::name_changed (GtkEditable*	editable,
    			           ChildIndex	child) {
    GtkWidget*      top_panel;
    NamedMenuEntry* entry;
    
    top_panel = GTK_WIDGET (editable)->parent;
    entry = (NamedMenuEntry*) gtk_object_get_data (GTK_OBJECT (top_panel), 
                                                   "NamedMenuEntry*");
    g_return_if_fail (top_panel);
    
    switch (child) {
    case EN_LABEL:
        g_free (entry->label);
	entry->label = g_strdup (gtk_entry_get_text (GTK_ENTRY (editable)));
	break;
    case EN_ICON:
        g_free (entry->icon);
	entry->icon = g_strdup (gtk_entry_get_text (GTK_ENTRY (editable)));
	entry->update_icon_button (get_children (top_panel) [BT_ICON]);
	break;
    default:
        g_assert (false);
    }

    Main::app.hierachy.update_tree_node ();
}
					 
GtkWidget* Submenu::get_editor (void) {
    GtkWidget *editor;
    
    editor = create_editor ();

    if (!top_panel) 
        gtk_widget_ref (top_panel = create_top_panel ());
	
    fill_top_panel (top_panel);
    set_top_panel (editor, top_panel);
    
    return editor;
}
