#include "propertynode.h"
#include "main.h"

IceWM_CM_NAMESPACE

PropertyNode* PropertyNode::current;

void PropertyNode::create_property_editor
					(const unsigned	index,
    					 GtkTable*	page,
    					 unsigned	row,
					 unsigned	left_attach,
					 unsigned	middle_attach,
					 unsigned	right_attach,
					 void		(*callback)(...)) {
    g_return_if_fail (index < prop_count ());
    g_return_if_fail (prop_editor (index) == NULL);
    
    switch (prop_def (index).type) {
    case PT_INTEGER: {
        GtkObject* adjustment =
	    gtk_adjustment_new (prop_def (index).defval.integer,
                                prop_def (index).min, prop_def (index).max, 
				1, 0, 0);
        gtk_table_attach (page, new_label (_(prop_def (index).label), 1.0, 1.0),
	                  left_attach, middle_attach, row, row + 1,
			  GTK_FILL, GTK_FILL, 0, 1);
        prop_editor (index) = gtk_hscale_new (GTK_ADJUSTMENT (adjustment));
        gtk_scale_set_digits (GTK_SCALE (prop_editor (index)), 0);
        gtk_signal_connect (adjustment, "value-changed", callback ? 
	                    callback : GTK_SIGNAL_FUNC (integer_changed),
		            (gpointer) index);
	break; }
    case PT_BOOLEAN: {
        prop_editor (index) =
	    gtk_check_button_new_with_label (_(prop_def (index).label));
        gtk_signal_connect (GTK_OBJECT (prop_editor (index)), "clicked", 
	                    callback ? callback 
			             : GTK_SIGNAL_FUNC (boolean_changed),
		            (gpointer) index);
	break; }
    case PT_STRING: {
        gtk_table_attach (page, new_label (_(prop_def (index).label)),
	                  left_attach, middle_attach, row, row + 1,
			  GTK_FILL, GTK_FILL, 0, 1);
			  
        prop_editor (index) = gtk_entry_new ();
        gtk_signal_connect (GTK_OBJECT (prop_editor (index)), "changed",
	                    callback ? callback
			             : GTK_SIGNAL_FUNC (string_changed),
		            (gpointer) index);
	break; }
    default:
        g_assert (false);
    }
    
    gtk_table_attach (page, prop_editor (index),
	              middle_attach, right_attach, row, row + 1,
	              GTK_FILL_AND_EXPAND, GTK_FILL, 0, 1);
}

void PropertyNode::synchronize_property_editor
					(const unsigned	index) {
    g_return_if_fail (index < prop_count ());
    g_return_if_fail (prop_editor (index) != NULL);
    
    switch (prop_def (index).type) {
    case PT_INTEGER:
        gtk_adjustment_set_value
            (gtk_range_get_adjustment (GTK_RANGE (prop_editor (index))),
	                               get_integer (index));
	break;
    case PT_BOOLEAN:
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prop_editor (index)), 
                                      get_boolean (index));
	break;
    case PT_STRING:
        gtk_entry_set_text (GTK_ENTRY (prop_editor (index)),
	                    get_string (index) ? get_string (index) : "");
	break;
    default:
        g_assert (false);
    }
}

void PropertyNode::synchronize_icon_button
					(GtkWidget*	icon_button) {
    GtkWidget*  old_icon;
    const Icon*	new_icon;
    
    if ((old_icon = GTK_BIN (icon_button)->child))
        gtk_container_remove (GTK_CONTAINER (icon_button), old_icon);
	
    new_icon = get_icon ();
    gtk_container_add (GTK_CONTAINER (icon_button),
             new_icon ? gtk_pixmap_new (new_icon->normal_pixmap, 
	                                new_icon->normal_bitmap)
	              : gtk_label_new (_("No icon")));
    gtk_widget_show_all (icon_button);
}

const TreeNode::Icon* PropertyNode::get_icon_from_prop
					(const unsigned index) {
    return (get_string (index) && *get_string (index)
           ? Main::app.load_icon (get_string (index)) : NULL);
}
    
void PropertyNode::integer_changed	(GtkAdjustment	*adjustment,
    					 const unsigned index) {
    g_return_if_fail (current_node () != NULL);
    current_node ()->set_property (index, (int) adjustment->value);
}

void PropertyNode::boolean_changed	(GtkButton	*button,
    					 const unsigned index) {
    g_return_if_fail (current_node () != NULL);
    current_node ()->set_property (index,
        gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)) == TRUE);
}

void PropertyNode::string_changed	(GtkEditable	*editable,
    					 const unsigned index) {
    g_return_if_fail (current_node () != NULL);
    current_node ()->set_property (index, 
                                   gtk_entry_get_text (GTK_ENTRY (editable)));
}
					 
int PropertyNode::get_integer		(const unsigned	index) {
    g_return_val_if_fail (index < prop_count (), 0);
    g_return_val_if_fail (prop_def (index).type == PT_INTEGER, 0);
    return property (index).integer;
}

bool PropertyNode::get_boolean		(const unsigned	index) {
    g_return_val_if_fail (index < prop_count (), false);
    g_return_val_if_fail (prop_def (index).type == PT_BOOLEAN, false);
    return property (index).boolean;
}

const char* PropertyNode::get_string	(const unsigned	index) {
    g_return_val_if_fail (index < prop_count (), NULL);
    g_return_val_if_fail (prop_def (index).type == PT_STRING, NULL);
    return property (index).string;
}

void PropertyNode::set_property		(const unsigned	index,
    					 const int	value) {
    g_return_if_fail (index < prop_count ());
    g_return_if_fail (prop_def (index).type == PT_INTEGER);
    property (index).integer = value;
}

void PropertyNode::set_property		(const unsigned	index,
    					 const bool	value) {
    g_return_if_fail (index < prop_count ());
    g_return_if_fail (prop_def (index).type == PT_BOOLEAN);
    property (index).boolean = value;
}

void PropertyNode::set_property		(const unsigned	index,
    					 char*		value) {
    g_return_if_fail (index < prop_count ());
    g_return_if_fail (prop_def (index).type == PT_STRING);
    
//    if (!has_default_value (index))
//        g_free (property (index).string);
    property (index).string = g_strdup (value);
}

bool PropertyNode::has_default_value	(const unsigned	index) {
    switch (prop_def (index).type) {
    case PT_INTEGER:
	return (prop_def (index).defval.integer == property (index).integer);
    case PT_BOOLEAN:
	return (prop_def (index).defval.boolean == property (index).boolean);
    case PT_STRING:
	return !strcmp (prop_def (index).defval.string ?
	                prop_def (index).defval.string : "",
	                property (index).string ?
			property (index).string : "");
    default:
	g_assert (false);
        return false;
    }
}

ostream& PropertyNode::write_value	(ostream&	os,
					 const unsigned	index) {
    switch (prop_def (index).type) {
    case PT_INTEGER:
        return os << property (index).integer;
    case PT_BOOLEAN:
        return os << property (index).boolean;
    case PT_STRING:
        return os << (property (index).string ? property (index).string : "");
    default:
	g_assert (false);
        return os;
    }
}
