#ifndef __PROPERTY_NODE_H__
#define __PROPERTY_NODE_H__

#include "../config.h"
#include "../treenode.h"
#include <iostream>

IceWM_CM_BEGIN_NAMESPACE

class PropertyNode : public TreeNode {
public:
    virtual ~PropertyNode		(void) { }

    enum PropType {
        PT_STRING,
	PT_INTEGER,
	PT_BOOLEAN
    };

    union PropValue {
	int				integer;
	bool				boolean;
	char*				string;
    };

    struct PropDef {
        unsigned			id;
	PropType			type;
	const char*			symbol;
	const char*			label;
	const char*			description;
	const PropValue			defval;
	int				min;
	int				max;
    };
    
private:
    static PropertyNode*		current;

public:
    int get_integer			(const unsigned	index);
    bool get_boolean			(const unsigned	index);
    const char* get_string		(const unsigned	index);

    void set_property			(const unsigned	index,
    					 const int	value);
    void set_property			(const unsigned	index,
    					 const bool	value);
    void set_property			(const unsigned	index,
    					 char*		value);
    void set_property			(const unsigned	index,
    					 const char*	value)
					{ set_property (index, (char*) value); }
					 
    void synchronize_icon_button	(GtkWidget*	icon_button);
    
protected:
    virtual unsigned prop_count		(void) = 0;
    virtual PropValue& property		(const unsigned	index) = 0;
    virtual PropDef& prop_def		(const unsigned	index) = 0;
    virtual GtkWidget*& prop_editor	(const unsigned	index) = 0;
    const Icon*	get_icon_from_prop	(const unsigned index);
    void init_properties		(void);
    
    void create_property_editor		(const unsigned	index,
    					 GtkTable*	page,
    					 unsigned	row,
					 unsigned	left_attach,
					 unsigned	middle_attach,
					 unsigned	right_attach,
					 void		(*callback)(...) = 0);
    void synchronize_property_editors	(void);
    void synchronize_property_editor	(const unsigned	index);

    static void integer_changed		(GtkAdjustment	*adjustment,
    					 const unsigned index);
    static void boolean_changed		(GtkButton	*button,
    					 const unsigned index);
    static void string_changed		(GtkEditable	*editable,
    					 const unsigned index);
					 
    PropValue& property_none		(void);
    bool has_default_value		(const unsigned	index);
    ostream& write_value		(ostream&	os,
					 const unsigned	index);
    
    static void grab_current		(PropertyNode*	node);
    static void ungrab_current		(PropertyNode*	node);
    static PropertyNode* current_node	(void) { return current; }
};

inline void PropertyNode::init_properties
					(void) {
    for (unsigned n = 0; n < prop_count (); ++n)
        property (n) = prop_def (n).defval;
}

inline void PropertyNode::synchronize_property_editors
					(void) {
    for (unsigned n = 0; n < prop_count (); ++n)
        synchronize_property_editor (n);
}

inline PropertyNode::PropValue& PropertyNode::property_none
					(void) {
    g_assert (false);
    static PropValue none;
    return none;
}

inline void PropertyNode::grab_current	(PropertyNode*	node) {
    g_return_if_fail (node != NULL);
    g_return_if_fail (current == node || current == NULL);
    current = node;
}

inline void PropertyNode::ungrab_current
					(PropertyNode*	node) {
    g_return_if_fail (current == node);
    current = NULL;
}

IceWM_CM_END_NAMESPACE

#endif
