#ifndef __TREE_NODE_H__
#define __TREE_NODE_H__

#include "config.h"
#include <iostream>

IceWM_CM_BEGIN_NAMESPACE

#define GTK_FILL_AND_EXPAND  ((GtkAttachOptions) (GTK_FILL|GTK_EXPAND))

class TreeNode {
public:
    struct Icon {
        GdkPixmap			*normal_pixmap;
        GdkBitmap			*normal_bitmap;
        GdkPixmap			*small_pixmap;
        GdkBitmap			*small_bitmap;
    };
    
    struct InsertionInfo {
        const char*			label;
        const char*			tooltip;
        const char*			tooltip_private;
        GtkSignalFunc			callback;
	gpointer			user_data;
    };
    
    virtual ~TreeNode			(void) { }

    virtual void create_sub_tree        (class Hierachy*	hierachy,
                                         GtkCTreeNode*		node) { }
    virtual const char* get_label	(void) = NULL;
    virtual const Icon* get_icon	(void) { return 0; }
    virtual const char* get_title	(void) { return get_label (); }
    virtual const char* get_description	(void) { return 0; }
    virtual GtkWidget* get_editor	(void) { return 0; }
    virtual void synchronize_content	(void) { }
    virtual bool is_leaf		(void) { return true; }
    virtual bool is_moveable		(void) { return false; }
    virtual InsertionInfo*
			insertion_list	(void) { return 0; }
    virtual bool is_removeable		(void) { return false; }
    virtual bool insert_expanded	(void) { return false; }
    
    static GtkWidget* new_label		(const char*		text,
    					 gfloat			xalign = 1.0,
    					 gfloat			yalign = 0.5);
};

inline GtkWidget* TreeNode::new_label	(const char*		text,
    					 gfloat			xalign,
    					 gfloat			yalign) {
    GtkWidget* label = gtk_accel_label_new (text);
    gtk_misc_set_alignment (GTK_MISC (label), xalign, yalign);
    return label;
}

IceWM_CM_END_NAMESPACE

#endif
