/*
 * Adopted from gdkrotated:
 * http://stat.auckland.ac.nz/~lyndon/gdkrotated.html
 */

#include "gtk90degree.h"

#define FREE free

static void gtk_90_degree_class_init	(Gtk90DegreeClass	*class);
static void gtk_90_degree_destroy	(GtkObject		*object);
static void gtk_90_degree_realize	(GtkWidget		*widget);
static gint gtk_90_degree_expose_event	(GtkWidget		*widget,
					 GdkEventExpose		*event);
static void gtk_90_degree_size_request	(GtkWidget		*widget,
					 GtkRequisition		*requisition);

GtkType gtk_90_degree_get_type () {
    static GtkType type_id = 0;

    if (!type_id) {
        GtkTypeInfo type_info = {
            "Gtk90Degree",
            sizeof (Gtk90Degree),
            sizeof (Gtk90DegreeClass),
            (GtkClassInitFunc) gtk_90_degree_class_init,
            (GtkObjectInitFunc) NULL,
            (GtkArgSetFunc) NULL,
            (GtkArgGetFunc) NULL
        };
        type_id = gtk_type_unique (gtk_widget_get_type (), &type_info);
    }
    return type_id;
}

static void gtk_90_degree_class_init (Gtk90DegreeClass *class) {
    GTK_OBJECT_CLASS (class)->destroy =		gtk_90_degree_destroy;
    GTK_WIDGET_CLASS (class)->realize =		gtk_90_degree_realize;
    GTK_WIDGET_CLASS (class)->expose_event =	gtk_90_degree_expose_event;
    GTK_WIDGET_CLASS (class)->size_request =	gtk_90_degree_size_request;
}

GtkWidget* gtk_90_degree_new (const char *text) {
    GtkWidget *widget;
    
    widget = gtk_widget_new (gtk_90_degree_get_type (), NULL);
    GTK_90_DEGREE (widget)->text = g_strdup (text);
    return widget;
}

static void gtk_90_degree_destroy (GtkObject *object) {
    GtkObjectClass *parent_class;

    g_free (GTK_90_DEGREE (object)->text);
    
    parent_class = 
        GTK_OBJECT_CLASS (gtk_type_parent_class (gtk_90_degree_get_type ()));
	
    if (parent_class->destroy) parent_class->destroy (object);
}

static void gtk_90_degree_realize (GtkWidget *widget) {
    GdkWindowAttr attributes;

    g_return_if_fail (widget != NULL);
    g_return_if_fail (GTK_IS_90_DEGREE (widget));

    GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);

    attributes.x = widget->allocation.x;
    attributes.y = widget->allocation.y;
    attributes.width = widget->allocation.width;
    attributes.height = widget->allocation.height;
    attributes.wclass = GDK_INPUT_OUTPUT;
    attributes.window_type = GDK_WINDOW_CHILD;
    attributes.event_mask = gtk_widget_get_events (widget) 
                          | GDK_EXPOSURE_MASK;
    attributes.visual = gtk_widget_get_visual (widget);
    attributes.colormap = gtk_widget_get_colormap (widget);

    widget->window = gdk_window_new (widget->parent->window, &attributes,
                   GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP);

    widget->style = gtk_style_attach (widget->style, widget->window);
    gdk_window_set_user_data (widget->window, widget);

    gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
}

static gint gtk_90_degree_expose_event	(GtkWidget	*widget,
					 GdkEventExpose	*event) {
    Gtk90Degree		*this;
    GtkRequisition	size;
    GtkStyle		*style;
    GdkColor		black, white;
    
    GdkPixmap		*pixmap;
    GdkGC		*offscreen, 
    			*gc_widget;
    GdkImage		*image;
    gint		x, y;

    this = GTK_90_DEGREE (widget);
    gtk_90_degree_size_request (widget, &size);
    style = gtk_widget_get_style (widget);
    
    black.pixel = gdk_rgb_xpixel_from_rgb (0);
    white.pixel = gdk_rgb_xpixel_from_rgb (0xffffffff);

    pixmap = gdk_pixmap_new (widget->window, size.height, size.width, 1);
    g_return_val_if_fail (pixmap != NULL, TRUE);
    
    offscreen = gdk_gc_new (pixmap);
    g_return_val_if_fail (offscreen != NULL, TRUE);
    
    gdk_gc_set_font (offscreen, style->font);
    gdk_gc_set_foreground (offscreen, &white);
    gdk_draw_rectangle (pixmap, offscreen, 1, 0, 0, size.height, size.width);
    gdk_gc_set_foreground (offscreen, &black);
    gdk_draw_string (pixmap, style->font, offscreen, 0, 
                     style->font->ascent, this->text);
    
    image = gdk_image_get (pixmap, 0, 0, size.height, size.width); 
    g_return_val_if_fail (image != NULL, TRUE);
         
    gc_widget = style->text_gc [GTK_WIDGET_STATE (widget)];

    for (y = 0; y < size.height; y++)
        for (x = 0; x < size.width; x++)
	    if (gdk_image_get_pixel (image, y, x) == black.pixel)
	        gdk_draw_point (widget->window, gc_widget, 
		size.width - x - 1, y);
		
    gdk_pixmap_unref (pixmap);
    gdk_gc_unref (offscreen);
    gdk_image_destroy (image);
    
    return FALSE;
}

static void gtk_90_degree_size_request	(GtkWidget	*widget,
					 GtkRequisition	*requisition) {
    Gtk90Degree	*this;
    GtkStyle	*style;
    
	
    this = GTK_90_DEGREE (widget);
    style = gtk_widget_get_style (widget);
    
    g_return_if_fail (requisition != NULL);
    g_return_if_fail (this->text != NULL);
    g_return_if_fail (style != NULL);
    
    requisition->width = (style->font->ascent + style->font->descent);
    requisition->height = gdk_string_width (style->font, this->text);
}
