#include "gtkhref.h"

#define FREE free

static void gtk_href_init		(GtkHRef		*href);
static void gtk_href_class_init		(GtkHRefClass		*class);
static void gtk_href_destroy		(GtkObject		*object);
static void gtk_href_realize		(GtkWidget		*widget);
static gint gtk_href_enter_notify_event	(GtkWidget		*widget,
                                         GdkEventCrossing	*event);
static gint gtk_href_leave_notify_event	(GtkWidget		*widget,
                                         GdkEventCrossing	*event);
static gint gtk_href_button_press_event	(GtkWidget		*widget,
                                         GdkEventButton		*event);
static gint gtk_href_button_release_event
					(GtkWidget		*widget,
                                         GdkEventButton		*event);

GtkType gtk_href_get_type () {
    static GtkType type_id = 0;

    if (!type_id) {
        GtkTypeInfo type_info = {
            "GtkHRef",
            sizeof (GtkHRef),
            sizeof (GtkHRefClass),
            (GtkClassInitFunc) gtk_href_class_init,
            (GtkObjectInitFunc) gtk_href_init,
            (GtkArgSetFunc) NULL,
            (GtkArgGetFunc) NULL
        };
        type_id = gtk_type_unique (gtk_event_box_get_type (), &type_info);
    }
    return type_id;
}

static void gtk_href_class_init (GtkHRefClass *class) {
    GTK_OBJECT_CLASS (class)->destroy = gtk_href_destroy;
    GTK_WIDGET_CLASS (class)->realize = gtk_href_realize;
    GTK_WIDGET_CLASS (class)->enter_notify_event = gtk_href_enter_notify_event;
    GTK_WIDGET_CLASS (class)->leave_notify_event = gtk_href_leave_notify_event;
    GTK_WIDGET_CLASS (class)->button_press_event = gtk_href_button_press_event;
    GTK_WIDGET_CLASS (class)->button_release_event =
                                                 gtk_href_button_release_event;
}

static void gtk_href_init (GtkHRef *href) {
    GTK_WIDGET_UNSET_FLAGS (href, GTK_CAN_FOCUS);
    gtk_container_add (GTK_CONTAINER (href), gtk_label_new (""));
    
    href->url = NULL;
}

GtkWidget* gtk_href_new (const char *url,
			 const char *label) {
    GtkWidget *widget;
    
    widget = gtk_widget_new (gtk_href_get_type (), NULL);
    gtk_href_set_label (GTK_HREF (widget), label ? label : url);
    gtk_href_set_url (GTK_HREF (widget), url);
    
    return widget;
}

static void gtk_href_destroy (GtkObject *object) {
    GtkObjectClass *parent_class =
        GTK_OBJECT_CLASS (gtk_type_parent_class (gtk_href_get_type ()));

    g_free (GTK_HREF (object)->url);
    if (parent_class->destroy) parent_class->destroy (object);
}

static void gtk_href_realize (GtkWidget	*widget) {
    GdkCursor      *cursor;
    GtkWidgetClass *parent_class =
        GTK_WIDGET_CLASS (gtk_type_parent_class (gtk_href_get_type ()));
    if (parent_class->realize) parent_class->realize (widget);
    
    gtk_widget_add_events (GTK_WIDGET (widget), GDK_ENTER_NOTIFY_MASK
                                              | GDK_LEAVE_NOTIFY_MASK
                                              | GDK_BUTTON_PRESS_MASK
                                              | GDK_BUTTON_RELEASE_MASK);
					      
    cursor = gdk_cursor_new (GDK_HAND2);
    gdk_window_set_cursor (widget->window, cursor);
    gdk_cursor_destroy (cursor);
}

const gchar* gtk_href_get_url (GtkHRef* href) {
    g_return_val_if_fail (GTK_IS_HREF (href), NULL);
    return href->url;
}

void gtk_href_set_label (GtkHRef*     href,
                         const gchar* label) {
    char *pattern, *str;
    
    g_return_if_fail (GTK_IS_HREF (href));
    
    pattern = g_strdup (label);
    for (str = pattern; *str; ++str) *str = '_';
    
    gtk_label_set_text (GTK_LABEL (GTK_BIN (href)->child), label);
    gtk_label_set_pattern (GTK_LABEL (GTK_BIN (href)->child), pattern);
    
    g_free (pattern);
}

void gtk_href_set_url (GtkHRef*     href,
                       const gchar* url) {
    g_return_if_fail (GTK_IS_HREF (href));
    g_free (href->url);
    href->url = g_strdup (url);
}

static gint gtk_href_enter_notify_event (GtkWidget		*widget,
                                         GdkEventCrossing	*event) {
    gint x = event->x, y = event->y;
    if (gdk_window_at_pointer (&x, &y) == widget->window)
        gtk_widget_set_state (GTK_BIN (widget)->child, 
                              GTK_WIDGET_HAS_GRAB (widget) ? 
			      GTK_STATE_ACTIVE : GTK_STATE_PRELIGHT);
    return TRUE;
}

static gint gtk_href_leave_notify_event (GtkWidget		*widget,
                                         GdkEventCrossing	*event) {
    gtk_widget_set_state (GTK_BIN (widget)->child, GTK_STATE_NORMAL);
    return TRUE;
}

static gint gtk_href_button_press_event	(GtkWidget	*widget,
                                         GdkEventButton	*event) {
    gtk_grab_add (widget);
    gdk_pointer_grab (widget->window, TRUE, GDK_BUTTON_RELEASE_MASK,
                      NULL, NULL, GDK_CURRENT_TIME);
    GTK_WIDGET_SET_FLAGS (widget, GTK_HAS_GRAB);
    gtk_widget_set_state (GTK_BIN (widget)->child, GTK_STATE_ACTIVE);
    return TRUE;
}

static gint gtk_href_button_release_event (GtkWidget		*widget,
                                           GdkEventButton	*event) {
    gtk_grab_remove (widget);
    gdk_pointer_ungrab (GDK_CURRENT_TIME);
    GTK_WIDGET_UNSET_FLAGS (widget, GTK_HAS_GRAB);
    if (GTK_WIDGET_STATE (GTK_BIN (widget)->child) == GTK_STATE_ACTIVE)
        g_message ("system (\"netscape -remote newwin \\\"%s\\\"\")",
	           GTK_HREF (widget)->url);
    gtk_widget_set_state (GTK_BIN (widget)->child, GTK_STATE_PRELIGHT);
    return TRUE;
}
