#include "gtkpeeredbuttons.h"

static void gtk_peered_buttons_init          (GtkPeeredButtons      *this);
static void gtk_peered_buttons_class_init    (GtkPeeredButtonsClass *class);
static void gtk_peered_buttons_destroy       (GtkObject             *object);
static void gtk_peered_buttons_connect_popup (GtkPeeredButtons      *this,
                                              GtkWidget             *button);
static void gtk_peered_buttons_popup         (GtkButton		    *button,
                                              gpointer		    data);
static gint gtk_peered_buttons_enter         (GtkWidget             *widget,
                                              GdkEventCrossing      *event,
			                      GtkWidget             *data);
static gint gtk_peered_buttons_leave         (GtkWidget             *widget,
                                              GdkEventCrossing      *event,
			                      GtkWidget             *data);
static gint gtk_peered_buttons_button_press  (GtkWidget             *widget,
                                              GdkEvent      	    *event,
			                      GtkWidget             *data);

/**
 * gtk_peered_buttons_get_type:
 *
 * Creates GtkPeeredButton's runtime type info and determinates 
 * a unique GtkType id.
 */
GtkType gtk_peered_buttons_get_type (void) {
    static GtkType type_id = 0;

    if (!type_id) {
        GtkTypeInfo type_info = {
            "GtkPeeredButtons",
            sizeof (GtkPeeredButtons),
            sizeof (GtkPeeredButtonsClass),
            (GtkClassInitFunc) gtk_peered_buttons_class_init,
            (GtkObjectInitFunc) gtk_peered_buttons_init,
            (GtkArgSetFunc) NULL,
            (GtkArgGetFunc) NULL
        };
        type_id = gtk_type_unique (gtk_hbox_get_type (), &type_info);
    }
    return type_id;
}

/**
 * gtk_peered_buttons_init:
 *
 * Initializes a GtkPeeredButton instance.
 *
 * @this:	the GtkPeeredButton instance
 */
static void gtk_peered_buttons_init (GtkPeeredButtons *this) {
    GtkWidget *border;

    this->relief = GTK_RELIEF_NORMAL;
    this->tooltips = NULL;
    
    gtk_box_pack_start (GTK_BOX (this), this->left = gtk_button_new (), 
                        FALSE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (this), this->right = gtk_button_new (),
                        FALSE, TRUE, 0);
    
    GTK_WIDGET_UNSET_FLAGS (this->left, GTK_CAN_FOCUS);
    GTK_WIDGET_UNSET_FLAGS (this->right, GTK_CAN_FOCUS);
    
    gtk_signal_connect (GTK_OBJECT (this->left), "enter-notify-event",
	                GTK_SIGNAL_FUNC (gtk_peered_buttons_enter), 
			this->right);
    gtk_signal_connect (GTK_OBJECT (this->left), "leave-notify-event",
	                GTK_SIGNAL_FUNC (gtk_peered_buttons_leave), 
			this->right);
    gtk_signal_connect (GTK_OBJECT (this->right), "enter-notify-event",
	                GTK_SIGNAL_FUNC (gtk_peered_buttons_enter), 
			this->left);
    gtk_signal_connect (GTK_OBJECT (this->right), "leave-notify-event",
	                GTK_SIGNAL_FUNC (gtk_peered_buttons_leave), 
			this->left);
			
    this->action_handler = 0;
    this->popup_handler = 0;
    gtk_peered_buttons_popup_left (this);
    
    this->popup = gtk_window_new (GTK_WINDOW_POPUP);
    gtk_window_set_policy (GTK_WINDOW (this->popup), FALSE, TRUE, TRUE);
    gtk_signal_connect (GTK_OBJECT (this->popup), "button-press-event",
	                GTK_SIGNAL_FUNC (gtk_peered_buttons_button_press), 
			this);
    gtk_widget_add_events (this->popup, GDK_BUTTON_PRESS_MASK);
    
    border = gtk_frame_new (NULL);
    gtk_frame_set_shadow_type (GTK_FRAME (border), GTK_SHADOW_OUT);
    gtk_container_add (GTK_CONTAINER (this->popup), border);
    
    this->box = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (border), this->box);
}

/**
 * gtk_peered_buttons_init_class:
 *
 * Initializes the virtual function table for GtkPeeredButton.
 *
 * @class:	GtkPeeredButton's virtual function table
 */
static void gtk_peered_buttons_class_init (GtkPeeredButtonsClass *class) {
    GtkObjectClass *object_class;
    
    object_class = GTK_OBJECT_CLASS (class);
    object_class->destroy = gtk_peered_buttons_destroy;
}

/**
 * gtk_peered_buttons_destroy:
 *
 * Cleans up GtkPeeredButton's waste of space after it's live time.
 * (virtual GtkPeeredButton::~GtkPeeredButton (void))
 *
 * @object:	our abandoned abdonedGtkPeeredButton instance
 */
static void gtk_peered_buttons_destroy (GtkObject *object) {
    GtkPeeredButtons *this;
    GtkObjectClass   *parent_class;
    
    this = GTK_PEERED_BUTTONS (object);
    
    if (this->tooltips)
        gtk_object_unref (GTK_OBJECT (this->tooltips));
	
    parent_class = 
        GTK_OBJECT_CLASS (gtk_type_parent_class (object->klass->type));
    if (parent_class->destroy) parent_class->destroy (object);
}

/**
 * gtk_peered_buttons_new:
 *
 * Allocates and initalizes a new instance of GtkPeeredButton.
 *
 * returns:	the GtkPeeredButton instance
 */
GtkWidget* gtk_peered_buttons_new (void) {
    return gtk_widget_new (gtk_peered_buttons_get_type (), NULL);
}

/**
 * gtk_peered_buttons_new_with_labels:
 *
 * Allocates and initalizes a new instance of GtkPeeredButton.
 * Additionally both buttons are labeled.
 *
 * @left:	the text for the left label
 * @right:	the text for the right label
 * returns:	the GtkPeeredButton instance
 */
GtkWidget* gtk_peered_buttons_new_with_labels (const gchar *left,
					       const gchar *right) {
    GtkWidget *widget;

    widget = gtk_peered_buttons_new ();
    gtk_container_add (GTK_CONTAINER (GTK_PEERED_BUTTONS (widget)->left),
                       gtk_label_new (left));
    gtk_container_add (GTK_CONTAINER (GTK_PEERED_BUTTONS (widget)->right),
                       gtk_label_new (right));

    return widget;
}

/**
 * gtk_peered_buttons_new_with_arrow_left:
 *
 * Allocates and initalizes a new instance of GtkPeeredButton.
 * The popup menu is assigned to left button which is labeled with
 * an array. The right label is labeled with the text specified in
 * @text
 *
 * @text:	the text for the label
 * @shadow:	the shadow style for the arrow
 * returns:	the GtkPeeredButton instance
 */
GtkWidget* gtk_peered_buttons_new_with_arrow_left (const gchar	  *text,
                                                   GtkShadowType shadow) {
    GtkWidget *widget;
    
    widget = gtk_peered_buttons_new ();
    gtk_peered_buttons_popup_left (GTK_PEERED_BUTTONS (widget));
    gtk_container_add (GTK_CONTAINER (GTK_PEERED_BUTTONS (widget)->left),
                       gtk_arrow_new (GTK_ARROW_DOWN, shadow));
    gtk_container_add (GTK_CONTAINER (GTK_PEERED_BUTTONS (widget)->right),
                       gtk_label_new (text));

    return widget;
}

/**
 * gtk_peered_buttons_new_with_arrow_right:
 *
 * Allocates and initalizes a new instance of GtkPeeredButton.
 * The popup menu is assigned to right button which is labeled with
 * an array. The left label is labeled with the text specified in
 * @text
 *
 * @text:	the text for the label
 * @shadow:	the shadow style for the arrow
 * returns:	the GtkPeeredButton instance
 */
GtkWidget* gtk_peered_buttons_new_with_arrow_right (const gchar	  *text,
                                                    GtkShadowType shadow) {
    GtkWidget *widget;
    
    widget = gtk_peered_buttons_new ();
    gtk_peered_buttons_popup_right (GTK_PEERED_BUTTONS (widget));
    gtk_container_add (GTK_CONTAINER (GTK_PEERED_BUTTONS (widget)->left),
                       gtk_label_new (text));
    gtk_container_add (GTK_CONTAINER (GTK_PEERED_BUTTONS (widget)->right),
                       gtk_arrow_new (GTK_ARROW_DOWN, shadow));

    return widget;
}

/**
 * gtk_peered_buttons_set_relief:
 *
 * Set the button style of the associated buttons.
 *
 * @this:	a instance of GtkPeeredButton 
 * @relief:	the button style to assign
 */
void gtk_peered_buttons_set_relief (GtkPeeredButtons *this,
				    GtkReliefStyle   relief) {
    g_return_if_fail (GTK_IS_PEERED_BUTTONS (this));
    
    this->relief = relief;
    
    gtk_button_set_relief (GTK_BUTTON (this->left), relief);
    gtk_button_set_relief (GTK_BUTTON (this->right), relief);
    
    gtk_container_foreach (GTK_CONTAINER (this->box),
                           (GtkCallback) &gtk_button_set_relief, 
			   (gpointer) relief);
}

/**
 * gtk_peered_buttons_popup_left:
 *
 * Assigned to the popup menu to the left button.
 *
 * @this:	a instance of GtkPeeredButton 
 */
void gtk_peered_buttons_popup_left (GtkPeeredButtons *this) {
    g_return_if_fail (GTK_IS_PEERED_BUTTONS (this));
    gtk_peered_buttons_connect_popup (this, this->left);
}

/**
 * gtk_peered_buttons_popup_right:
 *
 * Assigned to the popup menu to the right button.
 *
 * @this:	a instance of GtkPeeredButton 
 */
void gtk_peered_buttons_popup_right (GtkPeeredButtons *this) {
    g_return_if_fail (GTK_IS_PEERED_BUTTONS (this));
    gtk_peered_buttons_connect_popup (this, this->right);
}

/**
 * gtk_peered_buttons_set_tooltips:
 *
 * Assignes another tooltip group to this instance of GtkPeeredButton.
 *
 * @this:	a instance of GtkPeeredButton
 */
void gtk_peered_buttons_set_tooltips (GtkPeeredButtons	*this,
				      GtkTooltips	*tooltips) {
    g_return_if_fail (GTK_IS_PEERED_BUTTONS (this));
    g_return_if_fail (GTK_IS_TOOLTIPS (tooltips));
    
    if (this->tooltips)
        gtk_object_unref (GTK_OBJECT (this->tooltips));
	
    gtk_object_ref (GTK_OBJECT (this->tooltips = tooltips));
}

/**
 * gtk_peered_buttons_set_default_action:
 *
 * Sets up the default action button (The one which does not trigger the
 * popup menu).
 *
 * @this:		a instance of GtkPeeredButton
 * @label:		a text for the button label
 * @tip_text:		a tooltip text for this widget, NULL if you want none
 * @tip_private:	???
 * @callback:		a callback for the "clicked" signal or NULL
 * @user_data:		extra information for the @callback
 */
void gtk_peered_buttons_set_default_action
					(GtkPeeredButtons	*this,
					 const gchar		*label,
					 const gchar		*tip_text,
					 const gchar		*tip_private,
					 GtkSignalFunc		callback,
					 gpointer		user_data) {
    GtkWidget *action_button;

    g_return_if_fail (GTK_IS_PEERED_BUTTONS (this));
    action_button = (this->popup_button == this->right ? this->left 
                                                       : this->right);

    if (this->action_handler)
        gtk_signal_disconnect (GTK_OBJECT (action_button),
	                       this->action_handler);

    gtk_label_set_text (GTK_LABEL (GTK_BIN (action_button)->child), label);
    
    if (!this->tooltips)
        gtk_peered_buttons_set_tooltips (this, gtk_tooltips_new ());
    gtk_tooltips_set_tip (this->tooltips, action_button, tip_text, tip_private);
    
    this->action_handler =
        gtk_signal_connect (GTK_OBJECT (action_button),
	                    "clicked", callback, user_data);
}

/**
 * gtk_peered_buttons_append_widget:
 *
 * Append another widget to the popup menu.
 *
 * @this:		a instance of GtkPeeredButton
 * @widget:		the widget to append
 * @tip_text:		a tooltip text for this widget, NULL if you want none
 * @tip_private:	???
 * returns:		the widget appended
 */
GtkWidget* gtk_peered_buttons_append_widget (GtkPeeredButtons	*this,
				             GtkWidget		*widget,
				             const gchar	*tip_text,
				             const gchar	*tip_private) {
    g_return_val_if_fail (GTK_IS_PEERED_BUTTONS (this), NULL);
    g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
    
    gtk_box_pack_start (GTK_BOX (this->box), widget, TRUE, TRUE, 0);
    
    if (tip_text) {
        if (!this->tooltips)
            gtk_peered_buttons_set_tooltips (this, gtk_tooltips_new ());
	
        gtk_tooltips_set_tip (this->tooltips, widget, tip_text, tip_private);
    }

    return widget;
}

/**
 * gtk_peered_buttons_append_item:
 *
 * Append another button to the popup menu and connects a click handler.
 *
 * @this:		a instance of GtkPeeredButton
 * @text:		a label for the new button
 * @tip_text:		a tooltip text for this button, NULL if you want none
 * @tip_private:	???
 * @callback:		a callback for the "clicked" signal or NULL
 * @user_data:		extra information for the @callback
 * returns:		the button appended
 */
GtkWidget* gtk_peered_buttons_append_item (GtkPeeredButtons	*this,
                                           const gchar		*text,
                                           const gchar		*tip_text,
                                           const gchar		*tip_private,
                                           GtkSignalFunc	callback,
                                           gpointer		user_data) {
    GtkWidget *widget;
    
    g_return_val_if_fail (GTK_IS_PEERED_BUTTONS (this), NULL);
    g_return_val_if_fail (text != NULL, NULL);
    
    widget = gtk_button_new_with_label (text);
    GTK_WIDGET_UNSET_FLAGS (widget, GTK_CAN_FOCUS);
    gtk_misc_set_alignment (GTK_MISC (GTK_BIN (widget)->child), 0.0, 0.5); 
    gtk_peered_buttons_append_widget (this, widget, tip_text, tip_private);
    gtk_button_set_relief (GTK_BUTTON (widget), this->relief);
    
    if (callback)
        gtk_signal_connect (GTK_OBJECT (widget), "clicked", 
	                    callback, user_data);
			    
    return widget;
}

/**
 * gtk_peered_buttons_connect_popup:
 *
 * Assigned to the popup menu to the specifed button.
 *
 * @this:	a instance of GtkPeeredButton
 * @button:	the button to connect with 
 */
static void gtk_peered_buttons_connect_popup (GtkPeeredButtons *this,
                                              GtkWidget        *button) {
    if (this->popup_handler)
        gtk_signal_disconnect (GTK_OBJECT (this->popup_button),
	                       this->popup_handler);

    this->popup_handler =
        gtk_signal_connect (GTK_OBJECT (this->popup_button = button),
	                    "clicked", gtk_peered_buttons_popup, this);
}

/**
 * gtk_peered_buttons_popup:
 *
 * Callback to popup up the menu.
 *
 * @button:	the button which emitted the signal
 * @data:	a pointer to a instance of GtkPeeredButton
 */
static void gtk_peered_buttons_popup (GtkButton	*button,
                                      gpointer	data) {
    GtkPeeredButtons *this;
    GtkRequisition rsize;
    
    this = GTK_PEERED_BUTTONS (data);
    if (GTK_WIDGET_VISIBLE (this->popup)) {
           gtk_widget_hide (this->popup);
	   gtk_grab_remove (this->popup);
           gdk_pointer_ungrab (GDK_CURRENT_TIME);
    } else {	   
        gint x, y;
	
        gdk_window_get_deskrelative_origin (this->left->window, &x, &y);
        y+= this->left->allocation.height;	
        gtk_widget_set_uposition (this->popup, x, y);
        gtk_widget_show_all (this->popup);

        gtk_widget_size_request (this->popup, &rsize);
        gtk_widget_set_usize (this->popup, 
	                      MAX (GTK_WIDGET (this)->allocation.width, 
			           rsize.width), 0);

        gtk_grab_add (this->popup);
        gdk_pointer_grab (this->popup->window, TRUE,
                          GDK_BUTTON_PRESS_MASK | 
                          GDK_BUTTON_RELEASE_MASK |
                          GDK_POINTER_MOTION_MASK, 
                          NULL, NULL, GDK_CURRENT_TIME);
    }
}

/**
 * gtk_peered_buttons_enter:
 *
 * Callback to detect intruding ;-) mouse pointers.
 *
 * @widget:	the widget which emitted the event signal
 * @event:	Gdk supplied event description
 * @data:	a pointer to a instance of GtkPeeredButton
 */
static gint gtk_peered_buttons_enter (GtkWidget             *widget,
                                      GdkEventCrossing      *event,
			              GtkWidget             *data) {
    gtk_widget_set_state (GTK_WIDGET (data), GTK_STATE_PRELIGHT);
    return TRUE;
}
				 
/**
 * gtk_peered_buttons_leave:
 *
 * Callback to detect leaving mouse pointers.
 *
 * @widget:	the widget which emitted the event signal
 * @event:	Gdk supplied event description
 * @data:	a pointer to a instance of GtkPeeredButton
 */
static gint gtk_peered_buttons_leave (GtkWidget             *widget,
                                      GdkEventCrossing      *event,
			              GtkWidget             *data) {
    gtk_widget_set_state (GTK_WIDGET (data), GTK_STATE_NORMAL);
    return TRUE;
}
				 
/**
 * gtk_peered_buttons_button_press:
 *
 * Callback to detect mouse buttons pressing our widget
 * (and not the button).
 *
 * @widget:	the widget which emitted the event signal
 * @event:	Gdk supplied event description
 * @data:	a pointer to a instance of GtkPeeredButton
 */
static gint gtk_peered_buttons_button_press (GtkWidget *widget,
                                             GdkEvent  *event,
			                     GtkWidget *data) {
    gtk_peered_buttons_popup (NULL, data);
    return TRUE;
}


void gtk_peered_buttons_remove_widget	(GtkPeeredButtons	*this,
					 GtkWidget		*widget) {
    g_return_if_fail (GTK_IS_PEERED_BUTTONS (this));
    gtk_container_remove (GTK_CONTAINER (this->box), widget);
}

void gtk_peered_buttons_remove_all_widgets (GtkPeeredButtons	*this) {
    GList* children;
    
    while ((children = gtk_container_children (GTK_CONTAINER (this->box))))
        gtk_container_remove (GTK_CONTAINER (this->box), 
	                      GTK_WIDGET (children->data));
}
