/*

wmclass.c, based upon xprop.c and dsimple.c

Copyright 2000        Mathias Hasselmann <Mathias.Hasselmann@gmx.de>

Copyright 1990, 1998  The Open Group

All Rights Reserved.

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of The Open Group shall
not be used in advertising or otherwise to promote the sale, use or
other dealings in this Software without prior written authorization
from The Open Group.

*/

#include <gdk/gdkprivate.h>
#include <X11/cursorfont.h>
#include <X11/Xmu/WinUtil.h>

#include "wmclass.h"

/**
 * get_window_class:
 *
 * Retrieves the WM_CLASS property for a window the user selected.
 * On success you have to free the XClassHint members res_name and
 * res_class by calling XFree.
 *
 * @gdk_window:		A mapped Gdk window, needed to resolve the
 *			applications X11 connection.
 * @class_hint:		A structure provided by X11 holding the WM_CLASS
 *			property.
 * returns:		TRUE on success.
 */

gboolean grab_wm_class			(GdkWindow*	gdk_window,
					 XClassHint*	class_hint) {
    int		status;
    Cursor	cursor;
    XEvent	event;
    Display*    display = ((GdkWindowPrivate*) gdk_window)->xdisplay;
    int		screen = DefaultScreen (display);
    Window	target_win = None,
    		root = RootWindow (display,screen);
    int		buttons = 0;

    
    /* Make the target cursor */
    cursor = XCreateFontCursor(display, XC_crosshair);

    /* Grab the pointer using target cursor, letting it room all over */
    status = XGrabPointer (display, root, False,
                           ButtonPressMask | ButtonReleaseMask, GrabModeSync,
                           GrabModeAsync, root, cursor, CurrentTime);
    g_return_val_if_fail (status == GrabSuccess, FALSE);

    /* Let the user select a window... */
    while ((target_win == None) || (buttons != 0)) {
        /* allow one more event */
        XAllowEvents (display, SyncPointer, CurrentTime);
        XWindowEvent (display, root, ButtonPressMask | ButtonReleaseMask,
	              &event);
        switch (event.type) {
        case ButtonPress:
            if (target_win == None) {
                target_win = event.xbutton.subwindow; /* window selected */
                if (target_win == None) target_win = root;
            }
            buttons++;
        break;
        case ButtonRelease:
            if (buttons > 0) /* there may have been some down before we started */
                buttons--;
            break;
        }
    }

    XUngrabPointer (display, CurrentTime);      /* Done with pointer */
    
    if (target_win != None) {
        Window       root;
        int          dummyi;
        unsigned int dummyu;

        if (XGetGeometry (display, target_win, &root, &dummyi, &dummyi,
                          &dummyu, &dummyu, &dummyu, &dummyu) &&
                          target_win != root)
            target_win = XmuClientWindow (display, target_win);

        return XGetClassHint (display, target_win, class_hint);
    }
    
    return FALSE;
}
