#include "winoptnode.h"
#include "main.h"

#include <gtk90degree.h>
#include <wmclass.h>

IceWM_CM_NAMESPACE

PropertyNode::PropDef WinOptNode::prop_df [] = {
    { WM_NAME, PT_STRING, NULL, _("WM Name:"),
      _("TODO"), { 0 } },
    { WM_CLASS, PT_STRING, NULL, _("WM Class:"),
      _("TODO"), { 0 } },
      
    { ICON, PT_STRING, "icon", _("Icon:"),
      _("The name of the icon."), { 0 } },
    { WORKSPACE, PT_INTEGER, "workspace", _("Default workspace:"),
      _("Default workspace for this window (number, counting from 0)."),
      { 0 }, 0, 0 },
    { ALL_WORKSPACES, PT_BOOLEAN, "allWorkspaces",
      _("Visible on all workspaces"),
      _("If set to 1, window will be visible on all workspaces."), { 0 } },
    { LAYER, PT_INTEGER, "layer", _("Default layer:"),
      _("Default layer for the window. Layer can be one of the following "
        "strings: Desktop, Below, Normal, OnTop, Dock, AboveDock, Menu."),
      { 4 }, 0, 16 },

    { D_TITLE_BAR, PT_BOOLEAN, "dTitleBar", _("Show title bar"), 
      _("If set to 0, window will not have a title bar."), { 1 } },
    { D_SYS_MENU, PT_BOOLEAN, "dSysMenu", _("Show system menu"),
      _("If set to 0, window will not have a system menu."), { 1 } },
    { D_BORDER,	PT_BOOLEAN, "dBorder", _("Show border"),
      _("If set to 0, window will not have a border."), { 1 } },
    { D_RESIZE,	PT_BOOLEAN, "dResize", _("Draw resizeable border"),
      _("If set to 0, window will not have a resize border."), { 1 } },
    { D_CLOSE,	PT_BOOLEAN, "dClose", _("Draw close button"),
      _("If set to 0, window will not have a close button."), { 1 } },
    { D_MINIMIZE, PT_BOOLEAN, "dMinimize", _("Draw minimize button"),
      _("If set to 0, window will not have a minimize button."), { 1 } },
    { D_MAXIMIZE, PT_BOOLEAN, "dMaximize", _("Draw maximize button."),
      _("If set to 0, window will not have a maximize button."), { 1 } },

    { F_MOVE, PT_BOOLEAN, "fMove", _("Is moveable"), 
      _("If set to 0, window will not be moveable."), { 1 } },
    { F_RESIZE,	PT_BOOLEAN, "fResize", _("Is resizeable"), 
      _("If set to 0, window will not be resizeable."), { 1 } },
    { F_CLOSE, PT_BOOLEAN, "fClose", _("Is closeable"), 
      _("If set to 0, window will not be closeable."), { 1 } },
    { F_MINIMIZE, PT_BOOLEAN, "fMinimize", _("Is minimizeable"),
      _("If set to 0, window will not be minimizeable."), { 1 } },
    { F_MAXIMIZE, PT_BOOLEAN, "fMaximize", _("Is maximizeable"),
      _("If set to 0, window will not be maximizeable."), { 1 } },
    { F_HIDE, PT_BOOLEAN, "fHide", _("Is hideable"),
      _("If set to 0, window will not be hideable."), { 1 } },
    { F_ROLLUP,	PT_BOOLEAN, "fRollup", _("Is shadeable"),
      _("If set to 0, window will not be shadeable."), { 1 } },

    { IGNORE_WIN_LIST, PT_BOOLEAN, "ignoreWinList", _("Ignore window list"),
      _("If set to 1, window will not appear in the window list."), { 0 } },
    { IGNORE_TASK_BAR, PT_BOOLEAN, "ignoreTaskBar", _("Ignore task bar"),
      _("If set to 1, window will not appear on the task bar."), { 0 } },
    { IGNORE_QUICK_SWITCH, PT_BOOLEAN, "ignoreQuickSwitch",
      _("Ignore QuickSwitch"), _("If set to 1, window will not be accessible "
        "using QuickSwitch feature (Alt+Tab).."), { 0 } },
	
    { FULL_KEYS, PT_BOOLEAN, "fullKeys", _("Full keys"), 
      _("If set to 1, the window manager leave more keys (Alt+F?) to the "
        "application.."), { 0 } },
    { NO_FOCUS_ON_APP_RAISE, PT_BOOLEAN, "noFocusOnAppRaise",
      _("No focus on raise"), _("if set to 1, window will not automatically "
        "get focus as application raises it."), { 0 } },
    { IGNORE_NO_FOCUS_HINT, PT_BOOLEAN, "ignoreNoFocusHint",
      _("Ignore no-focus-hint"), _("If set to 1, icewm will focus even if "
        "the window does not handle input."), { 0 } },
};

GtkWidget* WinOptNode::prop_ed [WinOptNode::PROPERTY_COUNT];

TreeNode::InsertionInfo WinOptNode::insertion_info [] = {
    { _("Window"), _("Inserts a new set of window settings"), 0,
      GTK_SIGNAL_FUNC (insert_new_window), &Main::app.toolbar },
    { 0, 0, 0, 0, 0 }
};

GtkWidget* WinOptNode::editor;
GtkWidget* WinOptNode::bt_icon;
GtkWidget* WinOptNode::lb_layer;
    
WinOptNode::WinOptNode			(const char*	wm_class,
    					 const char*	wm_name) {
    g_assert (wm_class);
        
    init_properties ();
    prop_val [WM_CLASS].string = g_strdup (wm_class);
    prop_val [WM_NAME].string = wm_name ? g_strdup (wm_name) : NULL;
}

WinOptNode::~WinOptNode			(void) {
    for (unsigned n = 0; n < PROPERTY_COUNT; ++n)
        if (prop_df [n].type == PT_STRING)
            g_free (prop_val [n].string);
}

GtkWidget* WinOptNode::get_editor	(void) {
    GtkWidget *editor = create_editor ();

    grab_current (this);
    synchronize_property_editors ();
    gtk_widget_set_sensitive (prop_ed [WM_CLASS], !has_name_prop ());
    gtk_widget_set_sensitive (prop_ed [WM_NAME], has_name_prop ());

    return editor;
}
    
GtkWidget* WinOptNode::create_editor	(void) {
    if (!editor) {
	GtkTable* page;
	
        gtk_widget_ref (editor = gtk_notebook_new ());
	gtk_notebook_set_tab_pos (GTK_NOTEBOOK (editor), GTK_POS_RIGHT);
	gtk_notebook_set_scrollable (GTK_NOTEBOOK (editor), TRUE);
	gtk_notebook_set_tab_border (GTK_NOTEBOOK (editor), 0);

        page = GTK_TABLE (gtk_table_new (2, 4, FALSE));
        gtk_table_set_col_spacings  (page, 4);
        gtk_container_set_border_width (GTK_CONTAINER (page), 5);
	gtk_notebook_append_page (GTK_NOTEBOOK (editor), GTK_WIDGET (page),
				  gtk_90_degree_new _("Generic"));
	
	GtkWidget* bt_grab;
	create_property_editor (WM_CLASS, page, 1, 1, 2, 3,
			        GTK_SIGNAL_FUNC (label_changed));
	create_property_editor (WM_NAME, page, 2, 1, 2, 3,
			        GTK_SIGNAL_FUNC (label_changed));
        gtk_table_attach (page, bt_grab = gtk_button_new_with_label (_("Grab")),
                          3, 4, 1, 2, GTK_FILL, GTK_SHRINK, 0, 1);
        gtk_signal_connect (GTK_OBJECT (bt_grab), "clicked",
			    GTK_SIGNAL_FUNC (cb_grab_wm_class), NULL);

        create_property_editor (ICON, page, 3, 1, 2, 3, 
	                        GTK_SIGNAL_FUNC (icon_changed));
        gtk_table_attach (page, bt_icon = gtk_button_new (),
                          3, 4, 3, 5, GTK_FILL, GTK_SHRINK, 4, 0);
        gtk_widget_set_usize (bt_icon, 56, 56);
	
	create_property_editor (WORKSPACE, page, 4, 1, 2, 3);
	create_property_editor (ALL_WORKSPACES, page, 5, 1, 2, 4);
	create_property_editor (LAYER, page, 6, 1, 2, 3,
			        GTK_SIGNAL_FUNC (layer_changed));
        gtk_table_attach (page, lb_layer = gtk_label_new (_("Normal")),
                          3, 4, 6, 7, GTK_FILL, GTK_FILL, 0, 1);

        page = GTK_TABLE (gtk_table_new (4, 5, FALSE));
        gtk_table_set_col_spacings  (page, 4);
        gtk_container_set_border_width (GTK_CONTAINER (page), 5);
	gtk_notebook_append_page (GTK_NOTEBOOK (editor), GTK_WIDGET (page),
				  gtk_90_degree_new _("Appearance"));
	
	for (int n = D_TITLE_BAR, row = 1; n < D_CLOSE; ++n, ++row)
	    create_property_editor (n, page, row, 1, 2, 3);
	for (int n = D_CLOSE, row = 1; n < F_MOVE; ++n, ++row)
	    create_property_editor (n, page, row, 3, 4, 5);
	    
        page = GTK_TABLE (gtk_table_new (3, 5, FALSE));
        gtk_table_set_col_spacings  (page, 4);
        gtk_container_set_border_width (GTK_CONTAINER (page), 5);
	gtk_notebook_append_page (GTK_NOTEBOOK (editor), GTK_WIDGET (page),
				  gtk_90_degree_new _("Behaviour"));
	
	for (int n = F_MOVE, row = 1; n < IGNORE_WIN_LIST; ++n, ++row)
	    create_property_editor (n, page, row, 1, 2, 3);
	for (int n = IGNORE_WIN_LIST, row = 1; n < FULL_KEYS; ++n, ++row)
	    create_property_editor (n, page, row, 3, 4, 5);
	for (int n = FULL_KEYS, row = 5; n < PROPERTY_COUNT; ++n, ++row)
	    create_property_editor (n, page, row, 3, 4, 5);
    }
    
    return editor;
}

void WinOptNode::label_changed		(GtkEditable	*editable,
    					 const unsigned index) {
    string_changed (editable, index);
    Main::app.hierachy.update_tree_node ();
}

void WinOptNode::icon_changed		(GtkEditable	*editable,
    					 const unsigned index) {
    string_changed (editable, index);
    current_node ()->synchronize_icon_button (bt_icon);
    Main::app.hierachy.update_tree_node ();
}

void WinOptNode::layer_changed		(GtkAdjustment	*adjustment,
    					 const unsigned index) {
    integer_changed (adjustment, index);
    
    switch ((unsigned) adjustment->value) {
    case LAYER_DESKTOP:
        gtk_label_set_text (GTK_LABEL (lb_layer), _("Desktop"));
	break;
    case LAYER_BELOW:
        gtk_label_set_text (GTK_LABEL (lb_layer), _("Below"));
	break;
    case LAYER_NORMAL:
        gtk_label_set_text (GTK_LABEL (lb_layer), _("Normal"));
	break;
    case LAYER_ON_TOP:
        gtk_label_set_text (GTK_LABEL (lb_layer), _("OnTop"));
	break;
    case LAYER_DOCK:
        gtk_label_set_text (GTK_LABEL (lb_layer), _("Dock"));
	break;
    case LAYER_ABOVE_DOCK:
        gtk_label_set_text (GTK_LABEL (lb_layer), _("AboveDock"));
	break;
    case LAYER_MENU:
        gtk_label_set_text (GTK_LABEL (lb_layer), _("Menu"));
	break;
    default:
        gtk_label_set_text (GTK_LABEL (lb_layer), "-");
	break;
    }
}

void WinOptNode::insert_new_window	(GtkWidget*	widget,
    					 gpointer	user_data) {
    g_message ("TODO...");
    
//    Main::app.hierachy.insert_interactively (new Separator ());
    Main::app.toolbar.close_menu (widget, insertion_info + 0, 
                                  _("New Window"));

    cb_grab_wm_class (widget, user_data);
}

void WinOptNode::cb_grab_wm_class	(GtkWidget*	widget,
    					 gpointer	user_data) {
    g_message ("TODO...");

    XClassHint class_hint;
    if (grab_wm_class (widget->window, &class_hint)) {
        g_message ("%s/%s", class_hint.res_name, class_hint.res_class);
	XFree (class_hint.res_name);
	XFree (class_hint.res_class);
    }
}

ostream& WinOptNode::store		(ostream&	os) {
    for (int n = ICON; n < PROPERTY_COUNT; ++n)
        if (!has_default_value (n)) {
	    if (has_name_prop ())
	        os << prop_val [WM_NAME].string << '.';
	    write_value (os << prop_val [WM_CLASS].string << '.'
	                    << prop_df [n].symbol << ": ", n) << endl;
        }			    
    return os;
}
