#ifndef __WINOPT_NODE_H__
#define __WINOPT_NODE_H__

#include "config.h"
#include "propertynode.h"

IceWM_CM_BEGIN_NAMESPACE

class WinOptNode : public PropertyNode {
public:
    WinOptNode				(const char*	wm_class,
    					 const char*	wm_name = NULL);
    virtual ~WinOptNode			(void);

    enum PropId {
    	WM_NAME, WM_CLASS,
        ICON, WORKSPACE, ALL_WORKSPACES, LAYER,
	D_TITLE_BAR, D_SYS_MENU, D_BORDER, D_RESIZE, D_CLOSE, D_MINIMIZE,
	D_MAXIMIZE,
	F_MOVE, F_RESIZE, F_CLOSE, F_MINIMIZE, F_MAXIMIZE, F_HIDE, F_ROLLUP,
	IGNORE_WIN_LIST, IGNORE_TASK_BAR, IGNORE_QUICK_SWITCH,
	FULL_KEYS, NO_FOCUS_ON_APP_RAISE, IGNORE_NO_FOCUS_HINT,
	PROPERTY_COUNT
    };
    
    enum Layer {
	LAYER_DESKTOP			= 0, 
	LAYER_BELOW			= 2, 
	LAYER_NORMAL			= 4, 
	LAYER_ON_TOP			= 6,
	LAYER_DOCK			= 8, 
	LAYER_ABOVE_DOCK		= 10, 
	LAYER_MENU			= 12
    };
    
    static PropDef			prop_df [PROPERTY_COUNT];
    static InsertionInfo		insertion_info [];
    
protected:
    PropValue				prop_val [PROPERTY_COUNT];
    static GtkWidget*			prop_ed [PROPERTY_COUNT];

    static GtkWidget*			editor;
    static GtkWidget*			bt_icon;
    static GtkWidget*			lb_layer;

    virtual unsigned prop_count		(void) { return PROPERTY_COUNT; }
    virtual PropValue& property		(const unsigned	index);
    virtual PropDef& prop_def		(const unsigned	index);
    virtual GtkWidget*& prop_editor	(const unsigned	index);
    
public:
    virtual const char* get_label	(void);
    virtual const char* get_title	(void);
    virtual const Icon* get_icon	(void);
    virtual GtkWidget* get_editor	(void);
    virtual void synchronize_content	(void) { ungrab_current (this); }
    
    ostream& store			(ostream&	os);
    
protected:
    GtkWidget* create_editor		(void);
					 
    static void label_changed		(GtkEditable	*editable,
    					 const unsigned index);
    static void icon_changed		(GtkEditable	*editable,
    					 const unsigned index);
    static void layer_changed		(GtkAdjustment	*adjustment,
    					 const unsigned index);

    static void insert_new_window	(GtkWidget*	widget,
    					 gpointer	user_data);
    static void cb_grab_wm_class	(GtkWidget*	widget,
    					 gpointer	user_data);

    bool has_name_prop			(void);
    virtual bool is_leaf		(void) { return has_name_prop (); }
    virtual bool is_removeable		(void) { return true; }
    virtual InsertionInfo* insertion_list
    					(void) { return insertion_info; }
};

inline PropertyNode::PropValue& WinOptNode::property
					(const unsigned	index) {
    g_assert (index < PROPERTY_COUNT);
    return prop_val [index];
}

inline PropertyNode::PropDef& WinOptNode::prop_def
					(const unsigned	index) {
    g_assert (index < PROPERTY_COUNT);
    return prop_df [index];
}

inline GtkWidget*& WinOptNode::prop_editor
					(const unsigned	index) {
    g_assert (index < PROPERTY_COUNT);
    return prop_ed [index];
}

inline const char* WinOptNode::get_label
					(void) {
    return has_name_prop () ? prop_val [WM_NAME].string
    			    : prop_val [WM_CLASS].string;
}

inline const char* WinOptNode::get_title
					(void) {
    return has_name_prop () ? _("Window Instance Options")
    			    : _("Window Class Options");
}

inline const TreeNode::Icon* WinOptNode::get_icon
					(void) {
    return PropertyNode::get_icon_from_prop (ICON);
}
    
inline bool WinOptNode::has_name_prop	(void) {
    return (prop_val [WM_NAME].string && *prop_val [WM_NAME].string);
}

IceWM_CM_END_NAMESPACE

#endif
