#! /usr/bin/perl -w

# -----------------------------------------------------------------------------
# Initialization
# -----------------------------------------------------------------------------
if ($#ARGV != 1) {
    print "Usage: gen_perfs.pl <IceWM sources> <IceWM/CM sources>\n";
    exit 1;
}

$cpp = "gcc -DCFGDEF -DCFGDESC -DFONTS_ADOBE -DCONFIG_TASKBAR -DCONFIG_TOOLTIP -E";
$IceWM = "$ARGV[0]";
$IceWM_CM = "$ARGV[1]";
$module = "prefsnode";
$sourcefile = "$IceWM_CM/src/$module.cc";
$headerfile = "$IceWM_CM/src/$module.h";

-f $sourcefile || die "Can't find $sourcefile\n";
-f $headerfile || die "Can't find $headerfile\n";

$index = 1;
$pref[0] = "123";

# -----------------------------------------------------------------------------
# Property classifications
# -----------------------------------------------------------------------------

%PREF_CLASS = (
    'clickFocus',				'FOCUS',
    'focusOnClickClient',			'FOCUS',
    'autoRaise',				'FOCUS',
    'delayPointerFocus',			'FOCUS',
    
    'quickSwitch',				'FOCUS',
    'quickSwitchToMinimized',			'FOCUS',
    'quickSwitchToHidden',			'FOCUS',
    'quickSwitchToAllWorkspaces',		'FOCUS',
    
    'raiseOnFocus',				'FOCUS',
    'raiseOnClickClient',			'FOCUS',
    'raiseOnClickTitleBar',			'FOCUS',
    'raiseOnClickButton',			'FOCUS',
    'raiseOnClickFrame',			'FOCUS',

    'focusOnMap',				'FOCUS',
    'focusOnMapTransient',			'FOCUS',
    'focusOnMapTransientActive',		'FOCUS',
    
    'strongPointerFocus',			'FOCUS',
    'grabRootWindow',				'FOCUS',
    'passFirstClickToClient',			'FOCUS',
    'pointerColormap',				'FOCUS',

    'smartPlacement',				'WINDOW_PLACEMENT',
    'manualPlacement',				'WINDOW_PLACEMENT',
    'minimizeToDesktop',			'WINDOW_PLACEMENT',
    'showMoveSizeStatus',			'WINDOW_PLACEMENT',
    'opaqueMove',				'WINDOW_PLACEMENT',
    'opaqueResize',				'WINDOW_PLACEMENT',
    'snapMove',					'WINDOW_PLACEMENT',
    'EdgeResistance',				'WINDOW_PLACEMENT',
    'snapDistance',				'WINDOW_PLACEMENT',
    'limitSize',				'WINDOW_PLACEMENT',
    'limitPosition',				'WINDOW_PLACEMENT',
    'sizeMaximized',				'WINDOW_PLACEMENT',
    'centerTransientsOnOwner',			'WINDOW_PLACEMENT',
    
    'showPopupsAbovePointer',			'BEHAVIOR',
    'win95keys',				'BEHAVIOR',
    'modMetaIsCtrlAlt',				'BEHAVIOR',
    'autoDetectGnome',				'BEHAVIOR',
    'clientMouseActions',			'BEHAVIOR',
    'confirmLogout',				'BEHAVIOR',
    'ClickMotionDistance',			'BEHAVIOR',
    
    'ClickMotionDelay',				'TIMINGS',
    'MultiClickTime',				'TIMINGS',
    'ToolTipDelay',				'TIMINGS',
    'ToolTipTime',				'TIMINGS',
    'MenuActivateDelay',			'TIMINGS',
    'SubmenuActivateDelay',			'TIMINGS',
    'pointerFocusDelay',			'TIMINGS',
    'autoRaiseDelay',				'TIMINGS',
    'autoHideDelay',				'TIMINGS',
    'edgeSwitchDelay',				'TIMINGS',
    'scrollBarStartDelay',			'TIMINGS',
    'scrollBarDelay',				'TIMINGS',
    'autoScrollStartDelay',			'TIMINGS',
    'autoScrollDelay',				'TIMINGS',
    
    'useMouseWheel',				'MOUSE_BUTTONS',
    'useRootButtons',				'MOUSE_BUTTONS',
    'buttonRaiseMask',				'MOUSE_BUTTONS',
    'rootWinMenuButton',			'MOUSE_BUTTONS',
    'rootWinListButton',			'MOUSE_BUTTONS',
    'rootMenuButton',				'MOUSE_BUTTONS',
    
    'autoReloadMenus',				'MENUS',
    'menuMouseTracking',			'MENUS',
    'replayMenuCancelClick',			'MENUS',
    'showThemesMenu',				'MENUS',
    
    'showTaskBar',				'TASK_BAR',
    'taskBarAutoHide',				'TASK_BAR',
    'taskBarDoubleHeight',			'TASK_BAR',
    'taskBarAtTop',				'TASK_BAR',
    'taskBarShowStartMenu',			'TASK_BAR',
    'taskBarShowWindowListMenu',		'TASK_BAR',
    'taskBarShowWorkspaces',			'TASK_BAR',
    'taskBarShowWindows',			'TASK_BAR',
    'taskBarShowAllWindows',			'TASK_BAR',
    
    'taskBarShowClock',				'TASK_BAR_APPLETS',
    'prettyClock',				'TASK_BAR_APPLETS',
    'fmtTime',					'TASK_BAR_APPLETS',
    'fmtDate',					'TASK_BAR_APPLETS',
    'taskBarShowMailboxStatus',			'TASK_BAR_APPLETS',
    'beepOnNewMail',				'TASK_BAR_APPLETS',
    'mailCheckDelay',				'TASK_BAR_APPLETS',
    'countMailMessages',			'TASK_BAR_APPLETS',
    'taskBarShowCPUStatus',			'TASK_BAR_APPLETS',
    'taskBarCPUSamples',			'TASK_BAR_APPLETS',
    'taskBarShowNetStatus',			'TASK_BAR_APPLETS',
    'netDevice',				'TASK_BAR_APPLETS',
    'taskBarShowApm',				'TASK_BAR_APPLETS',
    
    'iconPath',					'PATHS',
    'mailBoxPath',				'PATHS',

    'mailCommand',				'PROGRAMS',
    'newMailCommand',				'PROGRAMS',
    'lockCommand',				'PROGRAMS',
    'clockCommand',				'PROGRAMS',
    'runDlgCommand',				'PROGRAMS',
    'openCommand',				'PROGRAMS',
    'terminalCommand',				'PROGRAMS',
    'logoutCommand',				'PROGRAMS',
    'logoutCancelCommand',			'PROGRAMS',
    'shutdownCommand',				'PROGRAMS',
    'rebootCommand',				'PROGRAMS',
    'cpuCommand',				'PROGRAMS',
    'netCommand',				'PROGRAMS',
    'addressBarCommand',			'PROGRAMS',
    
    'warpPointer',				'WORKSPACES',
    'edgeWorkspaceSwitching',			'WORKSPACES',
    
    'themeName',				'THEME',
    'themeAuthor',				'THEME',
    'themeDescription',				'THEME',
    'titleButtonsSupported',			'THEME',
    
    'centerBackground',				'DESKTOP_BACKGROUND',
    'DesktopBackgroundColor',			'DESKTOP_BACKGROUND',
    'DesktopBackgroundPixmap',			'DESKTOP_BACKGROUND',
    
    'titleButtonsLeft',				'TITLE_BAR',
    'titleButtonsRight',			'TITLE_BAR',
    'showFrameIcon',				'TITLE_BAR',
    'titleMaximizeButton',			'TITLE_BAR',
    'titleRollupButton',			'TITLE_BAR',
    'titleBarCentered',				'TITLE_BAR',
    
    'wsTitleBar',				'DECORATIONS',
    'wsCornerX',				'DECORATIONS',
    'wsCornerY',				'DECORATIONS',
    'wsBorderX',				'DECORATIONS',
    'wsBorderY',				'DECORATIONS',
    'wsDlgBorderX',				'DECORATIONS',
    'wsDlgBorderY',				'DECORATIONS',
    
    'clrDialog',				'COLORS',
    'clrActiveBorder',				'COLORS',
    'clrInactiveBorder',			'COLORS',
    'clrNormalTitleButton',			'COLORS',
    'clrNormalTitleButtonText',			'COLORS',
    'clrActiveTitleBar',			'COLORS',
    'clrInactiveTitleBar',			'COLORS',
    'clrActiveTitleBarText',			'COLORS',
    'clrInactiveTitleBarText',			'COLORS',
    'clrNormalMinimizedWindow',			'COLORS',
    'clrNormalMinimizedWindowText',		'COLORS',
    'clrActiveMinimizedWindow',			'COLORS',
    'clrActiveMinimizedWindowText',		'COLORS',
    'clrNormalMenu',				'COLORS',
    'clrActiveMenuItem',			'COLORS',
    'clrActiveMenuItemText',			'COLORS',
    'clrNormalMenuItemText',			'COLORS',
    'clrDisabledMenuItemText',			'COLORS',
    'clrMoveSizeStatus',			'COLORS',
    'clrMoveSizeStatusText',			'COLORS',
    'clrQuickSwitch',				'COLORS',
    'clrQuickSwitchText',			'COLORS',
    'clrDefaultTaskBar',			'COLORS',
    'clrNormalButton',				'COLORS',
    'clrNormalButtonText',			'COLORS',
    'clrActiveButton',				'COLORS',
    'clrActiveButtonText',			'COLORS',
    'clrNormalTaskBarApp',			'COLORS',
    'clrNormalTaskBarAppText',			'COLORS',
    'clrActiveTaskBarApp',			'COLORS',
    'clrActiveTaskBarAppText',			'COLORS',
    'clrMinimizedTaskBarApp',			'COLORS',
    'clrMinimizedTaskBarAppText',		'COLORS',
    'clrInvisibleTaskBarApp',			'COLORS',
    'clrInvisibleTaskBarAppText',		'COLORS',
    'clrScrollBar',				'COLORS',
    'clrScrollBarArrow',			'COLORS',
    'clrScrollBarSlider',			'COLORS',
    'clrListBox',				'COLORS',
    'clrListBoxText',				'COLORS',
    'clrListBoxSelected',			'COLORS',
    'clrListBoxSelectedText',			'COLORS',
    'clrToolTip',				'COLORS',
    'clrToolTipText',				'COLORS',
    'clrClock',					'COLORS',
    'clrClockText',				'COLORS',
    'clrApm',					'COLORS',
    'clrApmText',				'COLORS',
    'clrInput',					'COLORS',
    'clrInputText',				'COLORS',
    'clrInputSelection',			'COLORS',
    'clrInputSelectionText',			'COLORS',
    'clrLabel',					'COLORS',
    'clrLabelText',				'COLORS',
    'clrCpuUser',				'COLORS',
    'clrCpuSys',				'COLORS',
    'clrCpuNice',				'COLORS',
    'clrCpuIdle',				'COLORS',
    'clrNetSend',				'COLORS',
    'clrNetReceive',				'COLORS',
    'clrNetIdle',				'COLORS',
    
    'titleFontName',				'FONTS',
    'menuFontName',				'FONTS',
    'statusFontName',				'FONTS',
    'switchFontName',				'FONTS',
    'normalButtonFontName',			'FONTS',
    'activeButtonFontName',			'FONTS',
    'normalTaskBarFontName',			'FONTS',
    'activeTaskBarFontName',			'FONTS',
    'minimizedWindowFontName',			'FONTS',
    'listBoxFontName',				'FONTS',
    'toolTipFontName',				'FONTS',
    'labelFontName',				'FONTS',
    'clockFontName',				'FONTS',
    'apmFontName',				'FONTS',

    'gKeyWinRaise',				'KEY_BINDINGS',
    'gKeyWinOccupyAll',				'KEY_BINDINGS',
    'gKeyWinLower',				'KEY_BINDINGS',
    'gKeyWinClose',				'KEY_BINDINGS',
    'gKeyWinRestore',				'KEY_BINDINGS',
    'gKeyWinPrev',				'KEY_BINDINGS',
    'gKeyWinNext',				'KEY_BINDINGS',
    'gKeyWinMove',				'KEY_BINDINGS',
    'gKeyWinSize',				'KEY_BINDINGS',
    'gKeyWinMinimize',				'KEY_BINDINGS',
    'gKeyWinMaximize',				'KEY_BINDINGS',
    'gKeyWinMaximizeVert',			'KEY_BINDINGS',
    'gKeyWinHide',				'KEY_BINDINGS',
    'gKeyWinRollup',				'KEY_BINDINGS',
    'gKeyWinMenu',				'KEY_BINDINGS',
    'gKeySysSwitchNext',			'KEY_BINDINGS',
    'gKeySysSwitchLast',			'KEY_BINDINGS',
    'gKeySysWinNext',				'KEY_BINDINGS',
    'gKeySysWinPrev',				'KEY_BINDINGS',
    'gKeySysWinMenu',				'KEY_BINDINGS',
    'gKeySysDialog',				'KEY_BINDINGS',
    'gKeySysMenu',				'KEY_BINDINGS',
    'gKeySysRun',				'KEY_BINDINGS',
    'gKeySysWindowList',			'KEY_BINDINGS',
    'gKeySysAddressBar',			'KEY_BINDINGS',
    'gKeySysWorkspacePrev',			'KEY_BINDINGS',
    'gKeySysWorkspaceNext',			'KEY_BINDINGS',
    'gKeySysWorkspacePrevTakeWin',		'KEY_BINDINGS',
    'gKeySysWorkspaceNextTakeWin',		'KEY_BINDINGS',
    'gKeySysWorkspace1',			'KEY_BINDINGS',
    'gKeySysWorkspace2',			'KEY_BINDINGS',
    'gKeySysWorkspace3',			'KEY_BINDINGS',
    'gKeySysWorkspace4',			'KEY_BINDINGS',
    'gKeySysWorkspace5',			'KEY_BINDINGS',
    'gKeySysWorkspace6',			'KEY_BINDINGS',
    'gKeySysWorkspace7',			'KEY_BINDINGS',
    'gKeySysWorkspace8',			'KEY_BINDINGS',
    'gKeySysWorkspace9',			'KEY_BINDINGS',
    'gKeySysWorkspace10',			'KEY_BINDINGS',
    'gKeySysWorkspace11',			'KEY_BINDINGS',
    'gKeySysWorkspace12',			'KEY_BINDINGS',
    'gKeySysWorkspace1TakeWin',			'KEY_BINDINGS',
    'gKeySysWorkspace2TakeWin',			'KEY_BINDINGS',
    'gKeySysWorkspace3TakeWin',			'KEY_BINDINGS',
    'gKeySysWorkspace4TakeWin',			'KEY_BINDINGS',
    'gKeySysWorkspace5TakeWin',			'KEY_BINDINGS',
    'gKeySysWorkspace6TakeWin',			'KEY_BINDINGS',
    'gKeySysWorkspace7TakeWin',			'KEY_BINDINGS',
    'gKeySysWorkspace8TakeWin',			'KEY_BINDINGS',
    'gKeySysWorkspace9TakeWin',			'KEY_BINDINGS',
    'gKeySysWorkspace10TakeWin',		'KEY_BINDINGS',
    'gKeySysWorkspace11TakeWin',		'KEY_BINDINGS',
    'gKeySysWorkspace12TakeWin',		'KEY_BINDINGS'
);

# -----------------------------------------------------------------------------
# Scan default preferences definition and description
# -----------------------------------------------------------------------------
open (IN, "$cpp $IceWM/src/default.h|") or die $!;
while (<IN>) {
    s/"\s+"//g;

# Default values
    (/^\s*bool\s+(\w*)\s*=\s*(true|false)/ &&
        ($PREF{$1}{type} = 'PT_BOOLEAN',
	 $PREF{$1}{default} = $2,
        ($PREF{$1}{index} || ($PREF{$1}{index} = $index++)))) ||
    (/^\s*unsigned\s+int\s+(\w*)\s*=\s*([0-9]+)/ &&
        ($PREF{$1}{type} = 'PT_INTEGER',
	 $PREF{$1}{default} = $2,
        ($PREF{$1}{index} || ($PREF{$1}{index} = $index++)))) ||
    (/^\s*const\s+char\s*\*\s+(\w*)\s*=\s*("[^"]*"|0)/ &&
        ($PREF{$1}{type} = 'PT_STRING',
	 $PREF{$1}{default} = "(int) $2",
        ($PREF{$1}{index} || ($PREF{$1}{index} = $index++)))) ||
# Descriptions and ranges
    (/^\s*{\s*("[^"]+")\s*,\s*\&\s*(\w+)\s*,\s*("[^"]*")\s*}/ &&
        ($PREF{$2}{symbol} = $1,
	 $PREF{$2}{description} = $3,
        ($PREF{$2}{index} || ($PREF{$2}{index} = $index++)))) ||
    (/^\s*{\s*("[^"]+")\s*,\s*&\s*(\w+)\s*,\s*([0-9]+)\s*,\s*([0-9()*]+)\s*,\s*("[^"]*")\s*}/ &&
        ($PREF{$2}{symbol} = $1,
         $PREF{$2}{min} = $3,
         $PREF{$2}{max} = $4,
	 $PREF{$2}{description} = $5,
        ($PREF{$2}{index} || ($PREF{$2}{index} = $index++)))) ||
    (/^\s*{\s*("[^"]+")\s*,\s*&\s*(\w+)\s*,\s*\w+\s*,\s*("[^"]*")\s*}/ &&
        ($PREF{$2}{symbol} = $1,
	 $PREF{$2}{description} = $3,
        ($PREF{$2}{index} || ($PREF{$2}{index} = $index++)))) ||
    (/^\s*{\s*("[^"]+")\s*,\s*&\s*(\w+)\s*,\s*("[^"]*")\s*}/ &&
        ($PREF{$2}{symbol} = $1,
	 $PREF{$2}{description} = $3,
        ($PREF{$2}{index} || ($PREF{$2}{index} = $index++))));
}
close (IN);

# -----------------------------------------------------------------------------
# Scan key bindings definition
# -----------------------------------------------------------------------------
open (IN, "$cpp $IceWM/src/bindkey.h|") or die $!;
while (<IN>) {
    /^WMKey\s+(\w+)\s*=\s*{\s*([^,]+),\s*([0-9 +]+)\s*,\s*("[^"]*")\s*,\s*\w+\s*}\s*;\s*$/ &&
        ($PREF{$1}{type} = 'PT_STRING',
	 $PREF{$1}{default} = "(int) $4",
	 $PREF{$1}{key} = $2,
	 $PREF{$1}{mod} = $3,
        ($PREF{$1}{index} || ($PREF{$1}{index} = $index++)));
}
close (IN);

# -----------------------------------------------------------------------------
# Sort definition and create enumeration key
# -----------------------------------------------------------------------------
$pref_count = 0;
foreach $id (keys %PREF) {
    $pref[$PREF{$id}{index} - 1] = $id;

    $enum = $id;
    $enum =~ s/^gKey/key/;
    $enum =~ s/(.)([A-Z][a-z])/$1_\l$2/g;
    $enum = uc ($enum);

    $PREF{$id}{enum} = $enum;
    
    defined ($PREF{$id}{symbol}) &&
    (++$pref_count) &&
    !defined ($PREF_CLASS{$id}) &&
    ($PREF_CLASS{$id} = 'NOT_CLASSIFIED');
}

# -----------------------------------------------------------------------------
# Create patch for $module.cc
# -----------------------------------------------------------------------------
$old_time = gmtime ((stat $sourcefile)[9]);
$new_time = gmtime (time ());
$lines = $index * 3 - 3;

print <<EndOfStream
--- src/$module.cc  $old_time
+++ src/$module.cc  $new_time
@@ -6,2 +6,$lines @@
 PropertyNode::PropDef PrefsNode::prop_df [] = {
+/*
+ * Unused preferences:
+ *
EndOfStream
;

foreach $id (keys %PREF) {
    print "+ *  $id\n" unless $PREF{$id}{symbol};
}

print "+ */\n";

foreach $id (@pref) {
    $enum = $PREF{$id}{enum};
    $type = $PREF{$id}{type};
    $symbol = $PREF{$id}{symbol};
    if ($symbol) {
        $label = $symbol;
        $label =~ s/^"(TaskBar|Color|Key)/"/;
        $label =~ s/^"Color/"/;
        $label =~ s/(Command|FontName)"$/"/;
        $label =~ s/([a-z0-9])([A-Z])/$1 $2/g;
        $label =~ s/([^" ])([A-Z][a-z])/$1 $2/g;
        $label =~ s/Mod Meta/ModMeta/ig;
        $label =~ s/Ctrl Alt/CtrlAlt/ig;
        $label =~ s/Win /Window /ig;
        $label =~ s/Dlg /Dialog /ig;
        $label =~ s/Multi /Multi/ig;
        $label =~ s/cpu/CPU/ig;
        $label =~ s/apm/APM/ig;
        $label =~ s/"$/:"/ if ('PT_BOOLEAN' ne $type);
	
        $desc = $PREF{$id}{description};
        $default = $PREF{$id}{default};
        ( $PREF{$id}{min} && ($min = $PREF{$id}{min})) ||
        ( $PREF{$id}{mod} && ($min = $PREF{$id}{mod})) || ($min = 0);
        ( $PREF{$id}{max} && ($max = $PREF{$id}{max})) ||
        ( $PREF{$id}{key} && ($max = $PREF{$id}{key})) || ($max = 0);

        $PREF{$id}{key} && $max =~ s/XK_/GDK_/;

        print "+    { $enum, $type, $symbol,\n";
        print "+      _($label), _($desc),\n";
        print "+      { $default }, $min, $max },\n";
    }	
}

$start = $lines + 9;
$end = $pref_count + 2;
print <<EndOfStream
 };
@@ -9,2 +$start,$end @@
 PrefsNode::PropClass PrefsNode::prop_cl [PrefsNode::PROPERTY_COUNT] = {
EndOfStream
;

foreach $id (@pref) {
    print "+    { $PREF{$id}{enum}, $PREF_CLASS{$id} },\n" if $PREF_CLASS{$id};
}

print " };\n";

# -----------------------------------------------------------------------------
# Create patch for $module.h
# -----------------------------------------------------------------------------
$old_time = gmtime ((stat $headerfile)[9]);
$new_time = gmtime (time ());

$line = 0;
$line[$line] = "+        ";
foreach $id (@pref) {
    if ($PREF{$id}{symbol}) {
        if (length ($line [$line]) + length ($PREF{$id}{enum}) > 78) {
            $line[$line] = "$line[$line]\n";
            $line[++$line] = "+        ";
        }
        $line[$line] = "$line[$line] $PREF{$id}{enum},";
    }
}
$line[$line] = "$line[$line]\n";
$lines = $line + 4;

print <<EndOfStream
--- src/$module.h  $old_time
+++ src/$module.h  $new_time
@@ -11,2 +11,$lines @@
     enum PropId {
EndOfStream
;

foreach (@line) { print; }

print <<EndOfStream
+         PROPERTY_COUNT
     };
EndOfStream
;
